//#define NU32_STANDALONE  // uncomment if program is standalone, not bootloaded
#include "NU32.h"          // plib.h, config bits, constants, funcs for startup and UART
#define DELAYTIME 40000000 // 40 million core clocks, or 1 second

void delay();

int Entered,Exited;

void __ISR(_EXTERNAL_0_VECTOR, IPL2SOFT) Ext0ISR(void) {
  Entered = ReadCoreTimer();  // record time ISR begins
  IFS0CLR = 1 << 3;           // clear the interrupt flag
  NU32LED2 = 1;               // turn off LED2
  Exited = ReadCoreTimer();   // record time ISR ends
}

void __ISR(_EXTERNAL_1_VECTOR, IPL7SRS) Ext1ISR(void) {
  Entered = ReadCoreTimer();
  IFS0CLR = 1 << 7;
  NU32LED2 = 0;               // turn on LED2
  Exited = ReadCoreTimer();
}

int main(void) {
  int dt;
  char msg[128];

  NU32_Startup(); // cache on, min flash wait, interrupts on, LED/button init, UART init
  INTDisableInterrupts();  // step 2: disable interrupts at CPU
  INTCONSET = 0x3;         // step 3: INT0 and INT1 trigger on rising edge
  IPC0CLR = 31 << 24;      // step 4: clear 5 priority and subp bits for INT0
  IPC0 |= 10 << 24;        // step 4: set INT0 to priority 2 subpriority 2
  IPC1CLR = 0x1F << 24;    // step 4: clear 5 priority and subp bits for INT1
  IPC1 |= 0x1C << 24;      // step 4: set INT1 to priority 7 subpriority 0
  IFS0bits.INT0IF = 0;     // step 5: clear INT0 flag status
  IFS0bits.INT1IF = 0;     // step 5: clear INT1 flag status
  IEC0SET = 0x88;          // step 6: enable INT0 and INT1 interrupts
  INTEnableSystemMultiVectoredInt();  // step 7: CPU enabled for mvec interrupts
  while(1) {
    delay();               // delay, so results sent back at reasonable rate
    WriteCoreTimer(0);     // start timing
    IFS0bits.INT0IF = 1;   // artificially set the INT0 interrupt flag
    while(IFS0bits.INT0IF);// wait until the interrupt clears the flag
    dt = ReadCoreTimer();  // get elapsed time
    sprintf(msg,"IPL2SOFT in %3d out %3d total %3d time %4d ns\r\n",Entered,Exited,dt,dt*25);
    NU32_WriteUART1(msg);  // send times to the host

    delay();               // same as above, except for INT1
    WriteCoreTimer(0);
    IFS0bits.INT1IF = 1;   // trigger INT1 interrupt
    while(IFS0bits.INT1IF);
    dt = ReadCoreTimer();
    sprintf(msg," IPL7SRS in %3d out %3d total %3d time %4d ns\r\n",Entered,Exited,dt,dt*25);
    NU32_WriteUART1(msg);
  }
}

void delay() {
  WriteCoreTimer(0);
  while(ReadCoreTimer()<DELAYTIME);
}
